@@ -6,7 +6,7 @@ subst - Greple module for text search and substitution
 
 =head1 VERSION
 
-Version 2.08
+Version 2.09
 
 =head1 SYNOPSIS
 
@@ -180,12 +180,25 @@ Created from following guideline document.
 
 =item B<--exdict> sccc2.dict
 
+=item B<--exdict> jtf-style-guide-3.dict
+
+=item B<--jtf-style-guide>
+
+Created from following guideline document.
+
+    JTF日本語標準スタイルガイド（翻訳用）
+    第3.0版
+    2019年8月20日
+    一般社団法人 日本翻訳連盟（JTF）
+    翻訳品質委員会
+    https://www.jtf.jp/jp/style_guide/pdf/jtf_style_guide.pdf
+
 =item B<--sccc2>
 
 Dictionary used for "C/C++ セキュアコーディング 第2版" published in
 2014.
 
-https://www.jpcert.or.jp/securecoding_book_2nd.html
+    https://www.jpcert.or.jp/securecoding_book_2nd.html
 
 =back
 
@@ -197,29 +210,34 @@ https://www.jpcert.or.jp/securecoding_book_2nd.html
     or
     $ curl -sL http://cpanmin.us | perl - App::Greple::subst
 
-=head1 LICENSE
-
-Copyright (C) Kazumasa Utashiro.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
 =head1 SEE ALSO
 
 L<https://github.com/kaz-utashiro/greple>
 
 L<https://github.com/kaz-utashiro/greple-subst>
 
+https://www.jtca.org/standardization/katakana_guide_3_20171222.pdf
+
+https://www.jtf.jp/jp/style_guide/styleguide_top.html,
+https://www.jtf.jp/jp/style_guide/pdf/jtf_style_guide.pdf
+
 =head1 AUTHOR
 
 Kazumasa Utashiro
 
+=head1 LICENSE
+
+Copyright (C) 2017-2020 Kazumasa Utashiro.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
 =cut
 
 
 package App::Greple::subst;
 
-our $VERSION = '2.08';
+our $VERSION = '2.09';
 
 use v5.14;
 use strict;
@@ -242,31 +260,13 @@ our @EXPORT_OK   = qw();
 use Carp;
 use Data::Dumper;
 use Text::ParseWords qw(shellwords);
-use Getopt::EX::Numbers;
-use Getopt::EX::Module; # to avoid error. why?
 use App::Greple::Common;
 use App::Greple::Pattern;
+use App::Greple::subst::Dict;
 
 use File::Share qw(:all);
 $ENV{GREPLE_SUBST_DICT} //= dist_dir 'App-Greple-subst';
 
-# oo interface
-our @ISA = 'App::Greple::Pattern';
-{
-    sub new {
-	my $class = shift;
-	die if @_ < 2;
-	my($pattern, $correct) = splice @_, 0, 2;
-	my $obj = $class->SUPER::new($pattern, @_);
-	$obj->correct($correct);
-	$obj;
-    }
-    sub correct {
-	my $obj = shift;
-	@_ ? $obj->{CORRECT} = shift : $obj->{CORRECT};
-    }
-}
-
 package App::Greple::subst::SmartString {
     use List::Util qw(any);
     sub new {
@@ -299,12 +299,13 @@ our $opt_ignore_space = 1;
 our $opt_warn_overlap = 1;
 our $opt_warn_include = 0;
 our $opt_stat_style = "default";
+our $opt_show_comment = 0;
 
 my $current_file;
 my $contents;
-my @fromto;
 my @subst_diffcmd;
 my $ignorechar_re;
+my $dict = new App::Greple::subst::Dict;
 
 sub debug {
     $debug = 1;
@@ -331,20 +332,7 @@ sub subst_initialize {
     }
 
     if (my $select = $opt_subst_select) {
-	my $max = @fromto;
-	my $numbers = Getopt::EX::Numbers->new(max => $max);
-	my @select = do {
-	    map  { $_ - 1 }
-	    sort { $a <=> $b }
-	    grep { $_ <= $max }
-	    map  { $numbers->parse($_)->sequence }
-	    split /,/, $select;
-	};
-	@fromto = do {
-	    my @tmp = (undef) x $max;
-	    @tmp[@select] = @fromto[@select];
-	    @tmp;
-	};
+	$dict->select($select);
     }
 }
 
@@ -390,11 +378,16 @@ my @match_list;
 sub subst_show_stat {
     my %arg = @_;
 
+    my @fromto = $dict->dictionary;
     my $from_max = max map { vwidth $_->string  } grep { defined } @fromto;
     my $to_max   = max map { vwidth $_->correct } grep { defined } @fromto;
 
     for my $i (0 .. $#fromto) {
 	my $p = $fromto[$i] // next;
+	if ($p->is_comment) {
+	    say $p->comment if $opt_show_comment;
+	    next;
+	}
 	my($from_re, $to) = ($p->string, $p->correct // '');
 
 	my $hash = $match_list[$i] // {};
@@ -431,11 +424,10 @@ sub subst_show_stat {
 }
 
 sub read_dict {
-    my $dict = shift;
+    my $dictfile = shift;
+    say $dictfile if $opt_dictname;
 
-    say $dict if $opt_dictname;
-
-    open DICT, $dict or die "$dict: $!\n";
+    open DICT, $dictfile or die "$dictfile: $!\n";
 
     local $_;
     my $flag = FLAG_REGEX;
@@ -443,13 +435,14 @@ sub read_dict {
     while (<DICT>) {
 	print if $opt_printdict;
 	chomp;
-	s/^\s*#.*//;
-	/\S/ or next;
-
+	if (not /^\s*[^#]/) {
+	    $dict->add_comment($_);
+	    next;
+	}
 	my @param = grep { not m{^//+$} } split ' ';
 	splice @param, 0, -2; # leave last one or two
 	my($pattern, $correct) = @param;
-	push @fromto, __PACKAGE__->new($pattern, $correct, flag => $flag);
+	$dict->add($pattern, $correct, flag => $flag);
     }
     close DICT;
 }
@@ -495,15 +488,19 @@ sub subst_search {
     $current_file = delete $arg{&FILELABEL} or die;
 
     my @matched;
-    for my $index (0 .. $#fromto) {
-	my $p = $fromto[$index] // next;
+    my $index = -1;
+    for my $p ($dict->dictionary) {
+	$index++;
+	$p // next;
+	next if $p->is_comment;
 	my($from_re, $to) = ($p->string, $p->correct // '');
 	my @match = match_regions pattern => $p->regex;
 	next if @match == 0 and $opt_check ne 'all';
+	my $hash = $match_list[$index] //= {};
 	my $callback = sub {
 	    my($ms, $me, $i, $matched) = @_;
 	    my $s = $matched =~ s/$ignorechar_re//gr;
-	    $match_list[$index]->{$s}++;
+	    $hash->{$s}++;
 	    my $format = @opt_format[ $i % @opt_format ];
 	    sprintf($format,
 		    ($opt_subst && $to ne '' && $s ne $to) ?
@@ -635,6 +632,7 @@ builtin remember!      $remember_data
 builtin warn-overlap!  $opt_warn_overlap
 builtin warn-include!  $opt_warn_include
 builtin ignore-space!  $opt_ignore_space
+builtin show-comment!  $opt_show_comment
 
 option default \
 	--prologue subst_initialize \
@@ -689,8 +687,8 @@ option --exdict  --dict $ENV{GREPLE_SUBST_DICT}/$<shift>
 option --exdictdir --prologue 'sub{ say "$ENV{GREPLE_SUBST_DICT}"; exit }'
 
 option --jtca-katakana-guide --exdict jtca-katakana-guide-3.dict
-
-option --sccc2 --exdict sccc2.dict
+option --jtf-style-guide     --exdict jtf-style-guide-3.dict
+option --sccc2               --exdict sccc2.dict
 
 option --dumpdict --printdict --prologue 'sub{exit}'
 
