@@ -47,7 +47,7 @@ my %functable = (
 	$_;
     },
     '1-2'  => sub {
-	s/[エェケゲセゼテデネヘベペメレヱ]\K[イィー]/[イィー]/g;
+	s/[エェケゲセゼテデネヘベペメレヱ]\K[イー]/[イー]/g;
 	s/[オォコゴソゾトドノホボポモロヲ]\Kー/[ウゥー]/g if $opt->{monomania};
 	$_;
     },
